// Generated by CoffeeScript 1.9.3
(function() {
  var assert, describe, ref,
    slice = [].slice;

  ref = require("./helpers"), assert = ref.assert, describe = ref.describe;

  describe("Functional programming functions", function(context) {
    var _, binary, compose, curry, deepEqual, flip, identity, noOp, partial, pipe, ternary, unary, variadic, wrap;
    deepEqual = function(a, b) {
      assert = require("assert");
      try {
        assert.deepEqual(a, b);
        return true;
      } catch (_error) {
        return false;
      }
    };
    context.test("deepEqual", function() {
      assert(deepEqual({
        a: 1,
        b: 2
      }, {
        b: 2,
        a: 1
      }));
      return assert(!deepEqual({
        a: 1,
        b: 2
      }, {
        a: 1,
        b: 1
      }));
    });
    noOp = function() {};
    identity = function(x) {
      return x;
    };
    wrap = function(x) {
      return function() {
        return x;
      };
    };
    curry = function(f) {
      var cf;
      return (cf = function(ax) {
        return function() {
          var bx, cx;
          bx = 1 <= arguments.length ? slice.call(arguments, 0) : [];
          cx = ax.concat(bx);
          if (cx.length < f.length) {
            return cf(cx);
          } else {
            return f.apply(null, cx);
          }
        };
      })([]);
    };
    context.test("curry", function() {
      var f, g;
      f = curry(function(x, y, z) {
        return {
          x: x,
          y: y,
          z: z
        };
      });
      assert(deepEqual(f(1, 2, 3), {
        x: 1,
        y: 2,
        z: 3
      }));
      g = f(3, 2);
      return assert(deepEqual(g(1), {
        x: 3,
        y: 2,
        z: 1
      }));
    });
    _ = {};
    partial = function() {
      var ax, f;
      f = arguments[0], ax = 2 <= arguments.length ? slice.call(arguments, 1) : [];
      return function() {
        var a, bx;
        bx = 1 <= arguments.length ? slice.call(arguments, 0) : [];
        bx = [].concat(bx);
        return f.apply(null, ((function() {
          var i, len, results;
          results = [];
          for (i = 0, len = ax.length; i < len; i++) {
            a = ax[i];
            results.push(a === _ ? bx.shift() : a);
          }
          return results;
        })()).concat(bx));
      };
    };
    context.test("partial", function() {
      var pow, square;
      pow = Math.pow;
      square = partial(pow, _, 2);
      return assert((square(3)) === 9);
    });
    flip = function(f) {
      switch (f.length) {
        case 1:
          return f;
        case 2:
          return function(y, x) {
            return f(x, y);
          };
        case 3:
          return function(z, y, x) {
            return f(x, y, z);
          };
        default:
          return function() {
            var ax;
            ax = 1 <= arguments.length ? slice.call(arguments, 0) : [];
            return f.apply(null, ax.reverse());
          };
      }
    };
    context.test("flip", function() {
      var pow, square;
      pow = Math.pow;
      square = (curry(flip(pow)))(2);
      return assert((square(3)) === 9);
    });
    compose = function() {
      var f, fx, g, i;
      fx = 2 <= arguments.length ? slice.call(arguments, 0, i = arguments.length - 1) : (i = 0, []), f = arguments[i++];
      if (fx.length === 0) {
        return f;
      } else {
        g = compose.apply(null, fx);
        return function() {
          var ax, fax, ref1;
          ax = 1 <= arguments.length ? slice.call(arguments, 0) : [];
          if (((ref1 = (fax = f.apply(null, ax))) != null ? ref1.then : void 0) != null) {
            return fax.then(g);
          } else {
            return g(fax);
          }
        };
      }
    };
    context.test("compose", function*() {
      var _parse, _stringify, clone, clone1, clone2, clone3, data, parse, promise, stringify;
      data = {
        foo: 1,
        bar: 2,
        baz: 3
      };
      parse = JSON.parse, stringify = JSON.stringify;
      clone = compose(parse, stringify);
      assert(deepEqual(clone(data), data));
      promise = require("when").promise;
      _stringify = function(x) {
        return promise(function(resolve) {
          return setTimeout((function() {
            return resolve(stringify(x));
          }), 100);
        });
      };
      _parse = function(s) {
        return promise(function(resolve) {
          return setTimeout((function() {
            return resolve(parse(s));
          }), 100);
        });
      };
      clone1 = compose(parse, _stringify);
      clone2 = compose(_parse, stringify);
      clone3 = compose(_parse, _stringify);
      assert(deepEqual((yield clone1(data)), data));
      assert(deepEqual((yield clone2(data)), data));
      return assert(deepEqual((yield clone3(data)), data));
    });
    pipe = flip(compose);
    context.test("pipe", function() {
      var clone, data, parse, stringify;
      data = {
        foo: 1,
        bar: 2,
        baz: 3
      };
      parse = JSON.parse, stringify = JSON.stringify;
      clone = pipe(stringify, parse);
      return assert(deepEqual(clone(data), data));
    });
    variadic = function(f) {
      return function(ax) {
        return f.apply(null, ax);
      };
    };
    unary = function(f) {
      return function(x) {
        return f(x);
      };
    };
    binary = function(f) {
      return function(x, y) {
        return f(x, y);
      };
    };
    ternary = function(f) {
      return function(x, y, z) {
        return f(x, y, z);
      };
    };
    return module.exports = {
      deepEqual: deepEqual,
      noOp: noOp,
      identity: identity,
      wrap: wrap,
      curry: curry,
      _: _,
      partial: partial,
      flip: flip,
      compose: compose,
      pipe: pipe,
      variadic: variadic,
      unary: unary,
      binary: binary,
      ternary: ternary
    };
  });

}).call(this);
